#include <idc.idc>

#define	DebugMessage(x)	\
	Message x

static FileGetDosHeader()
{
	auto lPos;

	lPos = MinEA();
	DebugMessage((" ~ Module start at 0x%08x with 0x%04x\n", lPos, Word(lPos)));

	if (Word(lPos) != 'ZM')
	{
		Message(" ! Error: DOS header not found.\n");
		Message(" * Use manual load option and load all sections.\n");
		return BADADDR;
	}

	DebugMessage((" ~ DOS header at 0x%08x\n", lPos));
	return lPos;
}

static FileGetNtHeader()
{
	auto lPos;

	lPos = FileGetDosHeader();
	if (lPos == BADADDR) return BADADDR;

	lPos = lPos + Dword(lPos + 0x3C); // .w_lfanew
	DebugMessage((" ~ NT header start at 0x%08x with 0x%04x\n", lPos, Word(lPos)));

	if (Word(lPos) != 'EP')
	{
		Message(" ! Error: NT header not found.\n");
		return BADADDR;
	}

	DebugMessage((" ~ NT header at 0x%08x\n", lPos));
	return lPos;
}

static FileGetNtOptionalHeader()
{
	auto lPos;

	lPos = FileGetNtHeader();
	if (lPos == BADADDR) return BADADDR;

	lPos = lPos + 0x18;

	DebugMessage((" ~ Nt optional header at 0x%08x, magic 0x%04x.\n", lPos, Word(lPos)));
	return lPos;
}

static FileIsPe32(Magic)
{
	return (Magic == 0x10B);
}

static FileIsPe32Plus(Magic)
{
	return (Magic == 0x20B);
}

static FileGetDirectories()
{
	auto lPos, lMagic, lOffs;

	lPos = FileGetNtOptionalHeader();
	if (lPos == BADADDR) return BADADDR;

	lMagic = Word(lPos);
	if (FileIsPe32(lMagic))
	{
		lOffs = 96;
	}
	else if (FileIsPe32Plus(lMagic))
	{
		lOffs = 112;
	}
	else
	{
		Message("! Error: Invalid file format.\n");
		return BADADDR;
	}

	lPos = lPos + lOffs;
	
	DebugMessage((" ~ Directories at 0x%08x\n", lPos));
	return lPos;
}

#define	FileDirectoryExport		0
#define	FileDirectoryImport		1
#define	FileDirectoryResources		2
#define	FileDirectoryExceptions		3
#define	FileDirectoryCertificate	4
#define	FileDirectoryRelocations	5
#define	FileDirectoryDebug		6
#define	FileDirectoryArchitecture	7
#define	FileDirectoryGlobalPtr		8
#define	FileDirectoryTls		9
#define	FileDirectoryConfig		10
#define	FileDirectoryBoundImport	11
#define	FileDirectoryIat		12
#define	FileDirectoryDelayedImport	13
#define	FileDirectoryClr		14

static FileGetDirectory(Directory)
{
	auto lPos;

	lPos = FileGetDirectories();
	if (lPos == BADADDR) return BADADDR;

	lPos = lPos + Directory * 8;
	DebugMessage((" ~ Directory %i at 0x%08x\n", Directory, lPos));
	return lPos;
}

static FileGetRelocations()
{
	auto lPos;

	lPos = FileGetDosHeader();
	if (lPos == BADADDR) return BADADDR;

	lPos = lPos + Dword(FileGetDirectory(FileDirectoryRelocations) + 0);

	DebugMessage((" ~ Relocations data at 0x%08x\n", lPos));
	return lPos;
}

static FileValidateRelocationsSizes()
{
	auto lPos, lBase, lBlock;
	auto lDirRelocs, lRealRelocs;

	lBase = FileGetDosHeader();
	if (lBase == BADADDR) return BADADDR;

	lPos = FileGetDirectory(FileDirectoryRelocations);
	if (lPos == BADADDR) return BADADDR;

	lDirRelocs = Dword(lPos + 4); // Size of the dir
	DebugMessage((" ~ Relocs.Size = 0x%08x\n", lDirRelocs));

	lPos = FileGetRelocations();
	if (lPos == BADADDR) return BADADDR;

	lRealRelocs = 0;
	do {

		lBlock = lPos + lRealRelocs;
		DebugMessage((" ~ Relocs.Block[0x%08x] = [0x%08x, 0x%08x]\n", lBlock, lBase + Dword(lBlock), Dword(lBlock + 4)));
		lRealRelocs = lRealRelocs + Dword(lBlock + 4);

	} while(Dword(lPos + lRealRelocs) != 0);
	DebugMessage((" ~ Relocs.Calc = 0x%08x\n", lRealRelocs));

	return lRealRelocs != lDirRelocs;
}

static main()
{
	if (FileValidateRelocationsSizes())
		Message(" ! Relocations invalid");
	else
		Message(" * Relocations valid");
}
